//+------------------------------------------------------------------+
//|                                                          iZZ.mq4 |
//|                      Copyright  2013, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window

#property indicator_buffers   3
#property indicator_color1    White
#property indicator_color2    Red
#property indicator_color3    Yellow
#property indicator_width1    2
#property indicator_width2    2
#property indicator_width3    4

#define PERIOD 24

extern bool Shift=true;
extern bool Level=true;

int BARS,MODE,SHIFT,H,L,S; 

double MAIN[],SIGNAL[],MARK[];

bool EQ(int value1,int value2)
{
   return(value1==value2);
}

bool OPEN()
{
   return(EQ(MODE,MODE_OPEN));
}

double O(int shift)
{
   return(iOpen(NULL,PERIOD_H1,shift));
}

datetime T(int shift)
{
   return(iTime(NULL,PERIOD_H1,shift));
}

bool BARS(int shift)
{
   return(shift<iBars(NULL,PERIOD_H1));
}

int HST(int shift)
{
   return(iHighest(NULL,PERIOD_H1,MODE,PERIOD,shift));
}

int LST(int shift)
{
   return(iLowest(NULL,PERIOD_H1,MODE,PERIOD,shift));
}

int MIN(int S1,int S2)
{
   if(S1<S2) return(S1); else return(S2);
}

int MAX(int S1,int S2)
{
   if(S1>S2) return(S1); else return(S2);
}

void RESET(int shift)
{
   if(OPEN()) MAIN[shift]=EMPTY_VALUE; else SIGNAL[shift]=EMPTY_VALUE;
}

void WRITE(int shift)
{
   if(OPEN()) MAIN[shift]=O(shift); else SIGNAL[shift]=O(shift);
}

void ZZ(int S1,int S2)
{
   if(EQ(S1,S2)) return; 
   for(int i=MIN(S1,S2)+1; i<MAX(S1,S2); i++) RESET(i);
   WRITE(S1); 
   WRITE(S2);
}

void NEXT(int shift)
{
   S=shift; H=H(S); L=L(S);
}

int H(int shift)
{
   int S1=HST(shift);
   for(int S2=HST(S1); S1<S2; S2=HST(S1)) S1=S2;
   return(S1);
}

int L(int shift)
{
   int S1=LST(shift);
   for(int S2=LST(S1); S1<S2; S2=LST(S1)) S1=S2;
   return(S1);
}

void UPPER()
{
   ZZ(H,L); ZZ(H,S); NEXT(L);
}

void LOWER()
{
   ZZ(H,L); ZZ(S,L); NEXT(H);
}

int MAIN()
{
   if(H<L) UPPER();
   if(L<H) LOWER();
}

void CLEAR()
{  
   for(int i=MAX(H,L)+1; BARS(i); i++) RESET(i);
}

void DRAW(int mode)
{
   MODE=mode; 
   NEXT(SHIFT);   
   while(S<BARS) MAIN(); 
   CLEAR();
}      

void REMARK()
{
   for(int i=SHIFT+1; BARS(i); i++) MARK[i]=SIGNAL[i];
}

bool TOTAL(int index)
{
   return(index<ObjectsTotal());
}

bool VLINE(int id)
{
   string N=ObjectName(id);
   if(EQ(ObjectType(N),OBJ_VLINE)) return(true);
   return(false);
}

bool HLINE(int id)
{
   string N=ObjectName(id);
   if(EQ(ObjectType(N),OBJ_HLINE)) return(true);
   return(false);
}

int VERTICAL()
{
   for(int i=0; TOTAL(i); i++) if(VLINE(i)) return(i);   
   return(EMPTY);
}

int HORIZONTAL()
{
   for(int i=0; TOTAL(i); i++) if(HLINE(i)) return(i);   
   return(EMPTY);
}

int BAR(datetime time)
{
   return(iBarShift(NULL,PERIOD_H1,time));
}

void OFFSET(int shift)
{
   MAIN[shift]=EMPTY_VALUE;
   SIGNAL[shift]=EMPTY_VALUE;
   MARK[shift]=EMPTY_VALUE;
}

void SHIFT()
{
   SHIFT=0;
   int ID=VERTICAL();
   if(ID<0) return;
   string VLN=ObjectName(ID);
   ObjectSet(VLN,OBJPROP_COLOR,Yellow);
   ObjectSet(VLN,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(VLN,OBJPROP_WIDTH,0);
   datetime T=ObjectGet(VLN,OBJPROP_TIME1);
   int S=BAR(T);
   if(S<0) return;
   for(int i=0; i<S; i++) OFFSET(i);
   SHIFT=S;
}

int MARKER()
{
   for(int i=0; BARS(i); i++) 
      if(MARK[i]!=EMPTY_VALUE) return(i);
   return(EMPTY);      
}

void LEVEL()
{
   int ID=HORIZONTAL();
   if(ID<0) return;
   int S=MARKER();
   if(S<0) return;
   string HLN=ObjectName(ID);
   ObjectSet(HLN,OBJPROP_COLOR,Yellow);
   ObjectSet(HLN,OBJPROP_STYLE,STYLE_DASH);
   ObjectSet(HLN,OBJPROP_WIDTH,0);  
   ObjectMove(HLN,0,T(S),O(S));
}

void start()
{
   if(Shift) SHIFT();
   DRAW(MODE_OPEN); 
   DRAW(MODE_VOLUME);   
   REMARK();
   if(Level) LEVEL();
}

void init()
{
   BARS=PERIOD_MN1/PERIOD_H1+1;
   IndicatorBuffers(3);
   SetIndexBuffer(0,MAIN);
   SetIndexStyle(0,DRAW_SECTION);
   SetIndexBuffer(1,SIGNAL);
   SetIndexStyle(1,DRAW_SECTION);
   SetIndexBuffer(2,MARK);
   SetIndexStyle(2,DRAW_ARROW);
   SetIndexArrow(2,SYMBOL_STOPSIGN);   
   IndicatorDigits(Digits);
   IndicatorShortName("iZZ");     
}